@@ -23,6 +23,9 @@
    24/01/01 - Segfault in some systems fixed, Thanks for Manfred Bartz
    02/06/01 - Beter system detection & fix bug in OBSD, Thanks for Rudolf Leitgeb
    09/19/01 - Another Segfault in some systems fixed, Thanks for Andreas Tirok
+   06/26/02 - Fix problem with maximum uid number - Thanks for Gerard van Wageningen
+   07/02/02 - Minor fixes
+   Nelson Murilo, nelson@pangeia.com.br
 */
 
 #if defined(SOLARIS2) || defined(__linux__)
@@ -52,26 +55,21 @@
 
 #ifdef __FreeBSD__
 #define WTMP_FILENAME "/var/log/wtmp"
+#define LASTLOG_FILENAME "/var/log/lastlog"
 #endif
 #ifdef __OpenBSD__
 #define WTMP_FILENAME "/var/log/wtmp"
+#define LASTLOG_FILENAME "/var/log/lastlog"
 #endif
 #ifndef WTMP_FILENAME
 #define WTMP_FILENAME "/var/adm/wtmp"
 #endif
-
-#ifdef __FreeBSD__
-#define LASTLOG_FILENAME "/var/log/lastlog"
-#endif
-#ifdef __OpenBSD__
-#define LASTLOG_FILENAME "/var/log/lastlog"
-#endif
 #ifndef LASTLOG_FILENAME
 #define LASTLOG_FILENAME "/var/adm/lastlog"
 #endif
 
-#define TRUE 1
-#define FALSE 0
+#define TRUE 1L
+#define FALSE 0L
 
 long total_wtmp_bytes_read=0;
 size_t wtmp_file_size;
@@ -92,13 +90,15 @@ void free_results(struct s_localpwd *);
 uid_t *localgetpwnam(struct s_localpwd *, char *);
 int getslot(struct s_localpwd *, uid_t);
 
+#define MAX_ID 99999
+
 int main(int argc, char*argv[]) {
 	int		fh_wtmp;
 	int		fh_lastlog;
 	struct lastlog	lastlog_ent;
 	struct utmp	utmp_ent;
-	int		userid[65535];
-	int		i, slot;
+	long		userid[MAX_ID];
+	long		i, slot;
 	int		status = 0;
 	long		wtmp_bytes_read;
 	struct stat	wtmp_stat;
@@ -129,7 +129,7 @@ int main(int argc, char*argv[]) {
 
 	signal(SIGALRM, read_status);
 	alarm(5);
-	for (i=0; i<65535; i++)
+	for (i=0; i<MAX_ID; i++)
 		userid[i]=FALSE;
 
 	if ((fh_lastlog=open(lastlogfile,O_RDONLY)) < 0) {
@@ -161,9 +161,18 @@ int main(int argc, char*argv[]) {
 	    total_wtmp_bytes_read+=wtmp_bytes_read;
 	    if ( !nonuser(utmp_ent) && strncmp(utmp_ent.ut_line, "ftp", 3) &&
 		 (uid=localgetpwnam(localpwd,utmp_ent.ut_name)) != NULL )
-		if ( userid[*uid] == FALSE ) {
+            {
+                if (*uid > MAX_ID)
+                {
+                   fprintf(stderr, "MAX_ID is %ld and current uid is %ld, please check\n\r", MAX_ID, *uid );
+                   exit (1);
+
+                }
+		if (!userid[*uid]) 
+                {     
 		    lseek(fh_lastlog, (long)*uid * sizeof (struct lastlog), 0);
-		    if ((wtmp_bytes_read = read(fh_lastlog, &lastlog_ent, sizeof (struct lastlog))) > 0)
+		    if ((wtmp_bytes_read = read(fh_lastlog, &lastlog_ent, sizeof (struct lastlog))) > 0) 
+                    { 
                         if (wtmp_bytes_read < sizeof(struct lastlog))
                         {
                            fprintf(stderr, "lastlog entry may be corrupted");
@@ -180,7 +189,9 @@ int main(int argc, char*argv[]) {
                            ++status;
                         }
                         userid[*uid]=TRUE;
+                    }
 		}
+           }      
 	}
 	printf("\n");
 	free_results(localpwd);
